@@ -450,6 +450,34 @@ apply_error_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	reduce_throw( rc );
 }
 
+/* Args for "search". 
+ */
+static BuiltinTypeSpot *search_args[] = { 
+	&string_spot 
+};
+
+/* Do "search".
+ */
+/*ARGSUSED*/
+static void
+apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
+{
+	char buf[MAX_STRSIZE];
+	PElement rhs;
+	char *fn;
+
+	/* Get string. 
+	 */
+	PEPOINTRIGHT( arg[0], &rhs );
+	(void) reduce_get_string( rc, &rhs, buf, MAX_STRSIZE );
+
+	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
+		reduce_throw( rc );
+
+	heap_string_new( rc->heap, fn, out );
+	IM_FREE( fn );
+}
+
 /* Args for "print". 
  */
 static BuiltinTypeSpot *print_args[] = { 
@@ -501,6 +529,7 @@ apply_expand_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 static BuiltinInfo builtin_table[] = {
 	/* Other.
 	 */
+	{ "search", FALSE, 1, &search_args[0], &apply_search_call },
 	{ "error", FALSE, 1, &error_args[0], &apply_error_call },
 	{ "print", FALSE, 1, &print_args[0], &apply_print_call },
 	{ "expand", FALSE, 1, &expand_args[0], &apply_expand_call },
