@@ -75,7 +75,6 @@ columnview_caption_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 
 /* Select all objects in menu's column.
  */
-/*ARGSUSED*/
 static void
 columnview_select_cb( GtkWidget *wid, GtkWidget *host, Columnview *cview )
 {
@@ -812,6 +811,10 @@ columnview_refresh( vObject *vobject )
                 set_tooltip( cview->updownb, _( "Fold the column down" ) );
 	}
 
+	/* Closed columns are hidden in NOEDIT mode.
+	 */
+	widget_visible( GTK_WIDGET( cview ), editable || col->open );
+
         /* Set caption edit.
          */
         if( cview->state == COL_EDIT ) {
