@@ -431,7 +431,7 @@ typedef struct {
 	IMAGE *in, *out;
 } LintraInfo;
 
-/* Define what we do for each band element type. Non-complex input, any
+/* Define what we do for each band element type. Non-complex input, uchar
  * output.
  */
 #define LOOP(IN) { \
@@ -452,7 +452,7 @@ typedef struct {
 	} \
 }
 
-/* Complex input, complex output. 
+/* Complex input, uchar output. 
  */
 #define LOOPCMPLX(IN) {\
         IN *p = (IN *) in; \
@@ -461,26 +461,14 @@ typedef struct {
         for( x = 0; x < sz; x++ ) { \
 		double t; \
 		\
-		t = a * p[0] + b; \
+		t = a * p[x << 1] + b; \
 		\
 		if( t > 255 ) \
 			t = 255; \
 		else if( t < 0 ) \
 			t = 0; \
 		\
-                q[0] = t; \
-		\
-		t = a * p[1]; \
-		\
-		if( t > 255 ) \
-			t = 255; \
-		else if( t < 0 ) \
-			t = 0; \
-		\
-                q[1] = t; \
-		\
-                q += 2; \
-                p += 2; \
+                q[x] = t; \
         } \
 }
 
