@@ -228,7 +228,7 @@ dump_expr( Expr *expr )
 	Symbol *sym = expr->sym;
 
 	printf( "expr (0x%x)->sym->name = \"%s\"\n", 
-		(unsigned int) expr, IOBJECT( sym )->name );
+		GPOINTER_TO_UINT( expr ), IOBJECT( sym )->name );
 	printf( "%s->row = %s\n", IOBJECT( sym )->name, 
 		expr->row?"(set)":"(not set)" );
 
@@ -269,7 +269,7 @@ dump_compile( Compile *compile )
 	char str[100];
 
 	printf( "compile (0x%x)->sym->name = \"%s\"\n", 
-		(unsigned int) compile, IOBJECT( sym )->name );
+		GPOINTER_TO_UINT( compile ), IOBJECT( sym )->name );
 
 	printf( "%s->class = %s\n", 
 		IOBJECT( sym )->name, bool_to_char( compile->is_klass ) );
@@ -488,7 +488,7 @@ print_element( int nsp, EType type, void *arg )
 
 	switch( type ) {
 	case ELEMENT_NOVAL:
-		printf( "no value (%d)\n", (int) arg );
+		printf( "no value (%d)\n", GPOINTER_TO_INT( arg ) );
 		break;
 
 	case ELEMENT_NODE:
@@ -514,11 +514,12 @@ print_element( int nsp, EType type, void *arg )
 		break;
 
 	case ELEMENT_CHAR:
-		printf( "char \"%c\"", (int)arg );
+		printf( "char \"%c\"", GPOINTER_TO_UINT( arg ) );
 		break;
 
 	case ELEMENT_BOOL:
-		printf( "bool \"%s\"", bool_to_char( (gboolean)arg ) );
+		printf( "bool \"%s\"", bool_to_char(
+			(gboolean) GPOINTER_TO_UINT( arg ) ) );
 		break;
 
 	case ELEMENT_BINOP:
@@ -538,6 +539,11 @@ print_element( int nsp, EType type, void *arg )
 		printf( "tag \"%s\"", (char*)arg ); 
 		break;
 
+	case ELEMENT_STATIC:
+		printf( "static string \"%s\"", 
+			((HeapStaticString*)arg)->text ); 
+		break;
+
 	case ELEMENT_ELIST:
 		printf( "empty-list []" ); 
 		break;
