@@ -44,7 +44,7 @@ const int imagepresent_snap_threshold = 10;
 /* Cursor shape in id for each state.
  */
 iWindowShape imagepresent_cursors[IMAGEMODEL_LAST] = {
-	IWINDOW_SHAPE_NONE,		/* IMAGEMODEL_SELECT */
+	IWINDOW_SHAPE_EDIT,		/* IMAGEMODEL_SELECT */
 	IWINDOW_SHAPE_MOVE, 		/* IMAGEMODEL_PAN */
 	IWINDOW_SHAPE_MAGIN,		/* IMAGEMODEL_MAGIN */
 	IWINDOW_SHAPE_MAGOUT,		/* IMAGEMODEL_MAGOUT */
@@ -87,8 +87,8 @@ imagepresent_destroy( GtkObject *object )
 	Imagepresent *ip = IMAGEPRESENT( object );
 	iImage *iimage = ip->imagemodel->iimage;
 
-	FREEFI( g_source_remove, ip->scroll_tid );
-	FREEF( iwindow_cursor_context_destroy, ip->cntxt );
+	IM_FREEF( g_source_remove, ip->scroll_tid );
+	IM_FREEF( iwindow_cursor_context_destroy, ip->cntxt );
 	DESTROY_GTK( ip->ruler_menu );
 	if( iimage ) 
 		iimage->views = g_slist_remove( iimage->views, ip );
@@ -191,7 +191,14 @@ imagepresent_realize( GtkWidget *widget )
 	iWindow *iwnd = IWINDOW( gtk_widget_get_toplevel( widget ) );	
 
 	if( !ip->cntxt ) 
-		ip->cntxt = iwindow_cursor_context_new( iwnd );
+		ip->cntxt = iwindow_cursor_context_new( iwnd, 
+			0, "imagepresent" );
+
+	/* Set initial state. _realize() is too late ... the _refresh() has
+	 * already happened.
+	 */
+	iwindow_cursor_context_set_cursor( ip->cntxt, 
+		imagepresent_cursors[ip->imagemodel->state] );
 
 	GTK_WIDGET_CLASS( parent_class )->realize( widget );
 }
@@ -379,7 +386,7 @@ imagepresent_scroll_event_cb( GtkWidget *widget,
 
 	if( ev->direction == GDK_SCROLL_UP || 
 		ev->direction == GDK_SCROLL_DOWN ) {
-		if( ev->state & GDK_SHIFT_MASK ) {
+		if( ev->state & GDK_CONTROL_MASK ) {
 			if( ev->direction == GDK_SCROLL_UP )
 				imagepresent_zoom_in_centre( ip ); 
 			else
@@ -387,7 +394,7 @@ imagepresent_scroll_event_cb( GtkWidget *widget,
 
 			handled = TRUE;
 		}
-		else if( ev->state & GDK_CONTROL_MASK ) {
+		else if( ev->state & GDK_SHIFT_MASK ) {
 			if( ev->direction == GDK_SCROLL_UP )
 				imagepresent_set_position( ip, 
 					visible->left + incr, visible->top );
@@ -459,20 +466,19 @@ imagepresent_vadj_changed_cb( GtkAdjustment *adj, Imagepresent *ip )
 static void
 imagepresent_floating_new( Imagepresent *ip, 
 	int left, int top, int width, int height,
-	gboolean frozen, RegionviewType type, iWindowShape shape,
+	gboolean frozen, RegionviewType type, RegionviewResize resize,
 	int x, int y )
 {
-	if( !ip->regionview ) {
-		ip->floating.left = left;
-		ip->floating.top = top;
-		ip->floating.width = width;
-		ip->floating.height = height;
-		ip->regionview = regionview_new( NULL, &ip->floating, ip );
-		ip->regionview->frozen = frozen;
-		ip->regionview->type = type;
-		ip->regionview->shape = shape;
-		regionview_attach( ip->regionview, x, y );
-	}
+	ip->floating.left = left;
+	ip->floating.top = top;
+	ip->floating.width = width;
+	ip->floating.height = height;
+	ip->regionview = regionview_new( NULL, &ip->floating, ip );
+	ip->regionview->frozen = frozen;
+	ip->regionview->type = type;
+	ip->regionview->resize = resize;
+
+	regionview_attach( ip->regionview, x, y );
 }
 
 static gint
@@ -491,7 +497,8 @@ imagepresent_hruler_event( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 				IMAGEMODEL_SELECT, NULL );
 			imagepresent_floating_new( ip,
 				0, 0, im->Xsize, 0,
-				TRUE, REGIONVIEW_HGUIDE, IWINDOW_SHAPE_BOTTOM,
+				TRUE, REGIONVIEW_HGUIDE,
+				REGIONVIEW_RESIZE_BOTTOM,
 				ev->button.x, ev->button.y );
 
 			/* The pointer will be grabbed for the drag on the
@@ -532,7 +539,8 @@ imagepresent_vruler_event( GtkWidget *widget, GdkEvent *ev, Imagepresent *ip )
 				IMAGEMODEL_SELECT, NULL );
 			imagepresent_floating_new( ip,
 				0, 0, 0, im->Ysize,
-				TRUE, REGIONVIEW_VGUIDE, IWINDOW_SHAPE_RIGHT,
+				TRUE, REGIONVIEW_VGUIDE,
+				REGIONVIEW_RESIZE_RIGHT,
 				ev->button.x, ev->button.y );
 			gdk_pointer_ungrab( ev->button.time );
 			handled = TRUE;
@@ -792,7 +800,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 			imagepresent_floating_new( ip,
 				ix, iy, 0, 0,
 				FALSE, REGIONVIEW_MARK, 
-					IWINDOW_SHAPE_BOTTOMRIGHT,
+					REGIONVIEW_RESIZE_BOTTOMRIGHT,
 				x, y );
 
 			handled = TRUE;
@@ -840,7 +848,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 		im = imageinfo_get( FALSE, conv->ii );
 		imagepresent_floating_new( ip,
 			ix, iy, 0, 0,
-			TRUE, REGIONVIEW_LINE, IWINDOW_SHAPE_BOTTOMRIGHT,
+			TRUE, REGIONVIEW_LINE, REGIONVIEW_RESIZE_BOTTOMRIGHT,
 			x, y );
 		handled = TRUE;
 		break;
@@ -851,7 +859,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 		im = imageinfo_get( FALSE, conv->ii );
 		imagepresent_floating_new( ip,
 			ix, iy, 0, 0,
-			TRUE, REGIONVIEW_BOX, IWINDOW_SHAPE_BOTTOMRIGHT,
+			TRUE, REGIONVIEW_BOX, REGIONVIEW_RESIZE_BOTTOMRIGHT,
 			x, y );
 		handled = TRUE;
 		break;
@@ -873,7 +881,7 @@ imagepresent_left_press( Imagepresent *ip, GdkEvent *ev, int x, int y )
 			ix,
 			iy + imagemodel->text_area.top,
 			im2->Xsize, im2->Ysize,
-			TRUE, REGIONVIEW_BOX, IWINDOW_SHAPE_EDIT,
+			TRUE, REGIONVIEW_BOX, REGIONVIEW_RESIZE_EDIT,
 			x, y );
 
 		handled = TRUE;
@@ -1026,7 +1034,14 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 
 			switch( ip->regionview->type ) {
 			case REGIONVIEW_MARK:
-				buf_appendf( &buf, "%s ", CLASS_MARK );
+				/* How annoying, this used to be called
+				 * "Point".
+				 */
+				if( row->ws->compatibility )
+					buf_appends( &buf, "Point " );
+				else
+					buf_appendf( &buf, "%s ", CLASS_MARK );
+
 				row_qualified_name( row, &buf );
 				buf_appendd( &buf, ip->floating.left );
 				buf_appendd( &buf, ip->floating.top );
@@ -1454,6 +1469,7 @@ imagepresent_init( Imagepresent *ip )
 	ip->paint_last_x = 0;
 	ip->paint_last_y = 0;
 	ip->regionviews = NULL;
+	ip->grabbed = NULL;
 
 	/* Make main imagedisplay table.
 	 */
@@ -1507,6 +1523,19 @@ imagepresent_init( Imagepresent *ip )
 	assert( GTK_IS_SCROLLBAR( bar ) );
 	GTK_WIDGET_UNSET_FLAGS( bar, GTK_CAN_FOCUS );
 
+	/* Need one menu per image window (could have a single menu for all
+	 * windows, but then we'd have to set the state of the toggle buttons
+	 * before mapping)
+	 */
+	ip->ruler_menu = popup_build( _( "Ruler menu" ) );
+	popup_add_tog( ip->ruler_menu, _( "Rulers In _mm" ), 
+		POPUP_FUNC( imagepresent_rulers_mm_cb ) );
+	popup_add_tog( ip->ruler_menu, _( "Show _Offset" ), 
+		POPUP_FUNC( imagepresent_rulers_offset_cb ) );
+	menu_add_sep( ip->ruler_menu );
+	popup_add_but( ip->ruler_menu, GTK_STOCK_CLOSE,
+		POPUP_FUNC( imagepresent_ruler_hide_cb ) );
+
 	/* Make rulers.
 	 */
 	ip->hrule = GTK_HRULER( gtk_hruler_new() );
@@ -1551,19 +1580,6 @@ imagepresent_init( Imagepresent *ip )
 	gtk_widget_show( GTK_WIDGET( ip->id ) );
 	gtk_widget_show( GTK_WIDGET( swin ) );
 
-	/* Need one menu per image window (could have a single menu for all
-	 * windows, but then we'd have to set the state of the toggle buttons
-	 * before mapping)
-	 */
-	ip->ruler_menu = popup_build( _( "Ruler menu" ) );
-	popup_add_tog( ip->ruler_menu, _( "Rulers in mm" ), 
-		POPUP_FUNC( imagepresent_rulers_mm_cb ) );
-	popup_add_tog( ip->ruler_menu, _( "Show offset" ), 
-		POPUP_FUNC( imagepresent_rulers_offset_cb ) );
-	menu_add_sep( ip->ruler_menu );
-	popup_add_but( ip->ruler_menu, GTK_STOCK_CLOSE,
-		POPUP_FUNC( imagepresent_ruler_hide_cb ) );
-
 	/* Set initial ruler visibility on first refresh from imagemodel.
 	 */
 }
@@ -1672,7 +1688,7 @@ imagepresent_scroll_start( Imagepresent *ip, int u, int v )
 void
 imagepresent_scroll_stop( Imagepresent *ip )
 {
-	FREEFI( g_source_remove, ip->scroll_tid );
+	IM_FREEF( g_source_remove, ip->scroll_tid );
 	ip->u = 0;
 	ip->v = 0;
 }
