@@ -48,7 +48,9 @@ typedef struct _Imageview {
 	Conversionview *cv;
 	Statusview *sv;
 	Paintboxview *pbv;
-	GtkItemFactory *ifac;
+
+	GtkActionGroup *action_group;
+	GtkUIManager *ui_manager;
 } Imageview;
 
 typedef struct _ImageviewClass {
