@@ -136,7 +136,6 @@ extern int statfs();
 #include "formula.h"
 #include "orderitem.h"
 #include "orderlist.h"
-#include "led.h"
 #include "doubleclick.h"
 
 /* Generated marshallers.
@@ -176,6 +175,12 @@ extern int statfs();
 #define STOCK_RECT "nip-rect"
 #define STOCK_MOVE "nip-move"
 #define STOCK_SELECT "nip-select"
+#define STOCK_LED_RED "nip-led-red"
+#define STOCK_LED_GREEN "nip-led-green"
+#define STOCK_LED_BLUE "nip-led-blue"
+#define STOCK_LED_YELLOW "nip-led-yellow"
+#define STOCK_LED_CYAN "nip-led-cyan"
+#define STOCK_LED_OFF "nip-led-off"
 
 /* How much we decompile for error messages.
  */
