@@ -731,8 +731,6 @@ main_x_init( int *argc, char ***argv )
 	printf( "X11 init\n" );
 #endif/*DEBUG*/
 
-	gtk_set_locale();
-	setlocale( LC_NUMERIC, "C" );
 	(void) call_string_filename( (call_string_fn) gtk_rc_add_default_file, 
 		"$VIPSHOME" IM_DIR_SEP_STR "share" IM_DIR_SEP_STR 
 		PACKAGE IM_DIR_SEP_STR "rc" IM_DIR_SEP_STR 
@@ -886,6 +884,15 @@ main( int argc, char *argv[] )
 	printf( "%s: startup timer zeroed ...\n", argv[0] );
 #endif /*DEBUG_TIME*/
 
+	/* Want numeric locale to be "C", so we have C rules for doing 
+	 * double <-> string (ie. no "," for decimal point).
+	 */
+	setlocale( LC_NUMERIC, "C" );
+
+	/* Make sure our LC_NUMERIC setting is not trashed.
+ 	 */
+	gtk_disable_setlocale();
+
 	/* Set localised application name.
 	 */
 	g_set_application_name( _( PACKAGE ) );
@@ -905,11 +912,6 @@ main( int argc, char *argv[] )
 	 */
 	g_type_init();
 
-	/* Want numeric locale to be "C", so we have C rules for doing 
-	 * double <-> string (ie. no "," for decimal point).
-	 */
-	setlocale( LC_NUMERIC, "C" );
-
 #ifdef DEBUG
 	printf( "main: sizeof( struct heap_node ) == %d\n",
 		sizeof( struct heap_node ) );
