@@ -37,6 +37,20 @@
 #include <gtkextra/gtksheet.h>
 #endif /*HAVE_GTK_EXTRA*/
 
+/* Limit non-gtksheet display size with these ... could be prefs?
+ */
+static const int matrixview_max_width = 10;
+static const int matrixview_max_height = 10;
+
+/* A HAVE_GTK_EXTRA we can use in expressions.
+ */
+static const gboolean have_gtk_extra = 
+#ifdef HAVE_GTK_EXTRA
+	TRUE;
+#else /*!HAVE_GTK_EXTRA*/
+	FALSE;
+#endif /*HAVE_GTK_EXTRA*/
+
 static GraphicviewClass *parent_class = NULL;
 
 static void
@@ -55,8 +69,8 @@ matrixview_destroy( GtkObject *object )
 
     	/* My instance destroy stuff.
     	 */
-    	FREEF( g_slist_free, matrixview->items );
-	FREE( matrixview->cell_text );
+    	IM_FREEF( g_slist_free, matrixview->items );
+	IM_FREE( matrixview->cell_text );
 
     	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -110,7 +124,7 @@ matrixview_scan( View *view )
     	Expr *expr = HEAPMODEL( matrix )->row->expr;
 
     	gboolean changed;
-    	int x, y, i;
+    	int x, y;
     	GSList *p;
 
 #ifdef DEBUG
@@ -142,14 +156,13 @@ matrixview_scan( View *view )
     		return( view );
     	}
 
-    	/* Loop thro' all matrix widgets.
+    	/* Loop thru' all matrix widgets.
     	 */
     	if( matrixview->items ) 
-    		for( i = 0, p = matrixview->items, y = 0; 
-    			y < matrixview->height; y++ )
-    			for( x = 0; x < matrixview->width; 
-    				x++, p = p->next, i++ ) {
+    		for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
+    			for( x = 0; x < matrixview->width; x++, p = p->next ) {
     				GtkWidget *item = GTK_WIDGET( p->data );
+				int i = x + y * matrix->width;
 
     				GtkWidget *entry;
 
@@ -175,10 +188,11 @@ matrixview_scan( View *view )
     	if( matrixview->sheet ) {
     		GtkSheet *sheet = GTK_SHEET( matrixview->sheet );
 
-    		for( i = 0, y = 0; y < matrixview->height; y++ )
-    			for( x = 0; x < matrixview->width; x++, i++ ) {
+    		for( y = 0; y < matrixview->height; y++ )
+    			for( x = 0; x < matrixview->width; x++ ) {
     				char *txt = gtk_sheet_cell_get_text(
     					sheet, y, x );
+				int i = x + y * matrix->width;
 
     				if( !matrixview_scan_string( txt,
     					&matrix->value[i], &changed ) ) {
@@ -206,8 +220,10 @@ static void
 matrixview_toggle_change_cb( GtkWidget *widget, Matrixview *matrixview )
 {
     	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
-
     	int pos = g_slist_index( matrixview->items, widget );
+	int x = pos % matrixview->width;
+	int y = pos / matrixview->width;
+	int i = x + y * matrix->width;
 
 #ifdef DEBUG
     	printf( "matrixview_toggle_change_cb\n" );
@@ -215,17 +231,17 @@ matrixview_toggle_change_cb( GtkWidget *widget, Matrixview *matrixview )
 
     	/* Cycle value.
     	 */
-    	switch( (int) matrix->value[pos] ) {
+    	switch( (int) matrix->value[i] ) {
     	case 0:
-    		matrix->value[pos] = 128.0;
+    		matrix->value[i] = 128.0;
     		break;
 
     	case 255:
-    		matrix->value[pos] = 0.0;
+    		matrix->value[i] = 0.0;
     		break;
 
     	default:
-    		matrix->value[pos] = 255.0;
+    		matrix->value[i] = 255.0;
     		break;
     	}
 
@@ -243,13 +259,13 @@ matrixview_toggle_build( Matrixview *matrixview )
     	int x, y;
     	int cx, cy;
 
-    	matrixview->table = gtk_table_new( matrix->height, matrix->width, 
-    		TRUE );
+    	matrixview->table = gtk_table_new( 
+		matrixview->height, matrixview->width, TRUE );
     	gtk_box_pack_start( GTK_BOX( matrixview->box ), 
     		matrixview->table, FALSE, FALSE, 0 );
 
     	/* Find the centre position, if there is one. We give this a special
-    	 * name; it is highlighted by our resource file.
+    	 * name; it is highlit by our resource file.
     	 */
     	cx = -1; cy = -1;
     	if( matrix->height & 0x1 )
@@ -259,8 +275,8 @@ matrixview_toggle_build( Matrixview *matrixview )
 
     	/* Build contents.
     	 */
-    	for( y = 0; y < matrix->height; y++ )
-    		for( x = 0; x < matrix->width; x++ ) {
+    	for( y = 0; y < matrixview->height; y++ )
+    		for( x = 0; x < matrixview->width; x++ ) {
     			GtkWidget *but;
 
     			but = gtk_button_new_with_label( "0" );
@@ -292,13 +308,16 @@ matrixview_slider_change_cb( Tslider *tslider, Matrixview *matrixview )
 {
     	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
     	int pos = g_slist_index( matrixview->items, tslider );
+	int x = pos % matrixview->width;
+	int y = pos / matrixview->width;
+	int i = x + y * matrix->width;
 
     	assert( pos >= 0 );
 
     	/* Install value.
     	 */
-    	if( matrix->value[pos] != tslider->svalue ) {
-    		matrix->value[pos] = tslider->svalue;
+    	if( matrix->value[i] != tslider->svalue ) {
+    		matrix->value[i] = tslider->svalue;
 
     		classmodel_update( CLASSMODEL( matrix ) );
     		symbol_recalculate_all();
@@ -310,17 +329,15 @@ matrixview_slider_change_cb( Tslider *tslider, Matrixview *matrixview )
 static void
 matrixview_slider_build( Matrixview *matrixview )
 {
-    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
-
     	int x, y;
 
-    	matrixview->table = gtk_table_new( matrix->height, matrix->width, 
-    		TRUE );
+    	matrixview->table = gtk_table_new( matrixview->height, 
+		matrixview->width, TRUE );
     	gtk_box_pack_start( GTK_BOX( matrixview->box ), 
     		matrixview->table, TRUE, TRUE, 0 );
 
-    	for( y = 0; y < matrix->height; y++ )
-    		for( x = 0; x < matrix->width; x++ ) {
+    	for( y = 0; y < matrixview->height; y++ )
+    		for( x = 0; x < matrixview->width; x++ ) {
     			Tslider *tslider = tslider_new();
 
     			tslider_set_conversions( tslider, NULL, NULL );
@@ -412,17 +429,15 @@ matrixview_text_build_scale_offset( Matrixview *matrixview )
 static void
 matrixview_text_build( Matrixview *matrixview )
 {
-    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
-
     	int x, y;
 
-    	matrixview->table = gtk_table_new( matrix->height, matrix->width, 
-    		TRUE );
+    	matrixview->table = gtk_table_new( matrixview->height, 
+		matrixview->width, TRUE );
     	gtk_box_pack_start( GTK_BOX( matrixview->box ), 
     		matrixview->table, FALSE, FALSE, 0 );
 
-    	for( y = 0; y < matrix->height; y++ )
-    		for( x = 0; x < matrix->width; x++ ) {
+    	for( y = 0; y < matrixview->height; y++ )
+    		for( x = 0; x < matrixview->width; x++ ) {
     			GtkWidget *txt;
 
     			txt = build_entry( 8 );
@@ -434,7 +449,7 @@ matrixview_text_build( Matrixview *matrixview )
     				g_slist_append( matrixview->items, txt );
     		}
 
-    	if( matrix->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET )
+    	if( matrixview->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET )
     		/* Make the scale/offset widgets too.
     		 */
     		matrixview_text_build_scale_offset( matrixview );
@@ -514,14 +529,14 @@ matrixview_text_event( GtkWidget *widget,
 static void
 matrixview_text_sheet_build( Matrixview *matrixview )
 {
-    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
 	GtkWidget *entry;
 	GtkAdjustment *hadj, *vadj;
 
     	int cell_width;
     	int cell_height;
 
-    	matrixview->sheet = gtk_sheet_new( matrix->height, matrix->width, "" );
+    	matrixview->sheet = gtk_sheet_new( matrixview->height, 
+		matrixview->width, "" );
     	gtk_sheet_hide_column_titles( GTK_SHEET( matrixview->sheet ) );
     	gtk_sheet_hide_row_titles( GTK_SHEET( matrixview->sheet ) );
     	gtk_sheet_set_selection_mode( GTK_SHEET( matrixview->sheet ), 
@@ -543,8 +558,8 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 
 	/* For large matricies, display in a scrolled window.
 	 */
-	if( matrix->width * cell_width > 500 || 
-		matrix->height * cell_height > 500 ) {
+	if( matrixview->width * cell_width > 500 || 
+		matrixview->height * cell_height > 500 ) {
 		matrixview->swin = gtk_scrolled_window_new( NULL, NULL );
 		gtk_scrolled_window_set_policy( 
 			GTK_SCROLLED_WINDOW( matrixview->swin ),
@@ -555,8 +570,8 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 			GTK_SCROLLED_WINDOW( matrixview->swin ) );
 
 		gtk_widget_set_size_request( GTK_WIDGET( matrixview->swin ), 
-			IM_MIN( 500, matrix->width * cell_width + 40 ), 
-			IM_MIN( 500, matrix->height * cell_height + 40 ) );
+			IM_MIN( 500, matrixview->width * cell_width + 40 ), 
+			IM_MIN( 500, matrixview->height * cell_height + 40 ) );
 
 		gtk_container_add( GTK_CONTAINER( matrixview->swin ), 
 			matrixview->sheet );
@@ -570,8 +585,8 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 			gtk_adjustment_new( 0, 0, 100, 10, 10, 10 ) );
 
 		gtk_widget_set_size_request( GTK_WIDGET( matrixview->sheet ), 
-			matrix->width * cell_width, 
-			matrix->height * cell_height );
+			matrixview->width * cell_width, 
+			matrixview->height * cell_height );
 
 		gtk_box_pack_start( GTK_BOX( matrixview->box ), 
 			matrixview->sheet, FALSE, FALSE, 0 );
@@ -584,14 +599,12 @@ matrixview_text_sheet_build( Matrixview *matrixview )
 static void
 matrixview_text_build( Matrixview *matrixview )
 {
-    	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
-
 	/* Sheet hates zero width/height. Can happen during class construct.
 	 */
-	if( matrix->width > 0 && matrix->height > 0 )
+	if( matrixview->width > 0 && matrixview->height > 0 )
 		matrixview_text_sheet_build( matrixview );
 
-    	if( matrix->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET )
+    	if( matrixview->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET )
     		matrixview_text_build_scale_offset( matrixview );
 }
 
@@ -634,9 +647,9 @@ matrixview_toggle_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			GtkWidget *wid = GTK_WIDGET( p->data );
-    			int pos = x + y * matrixview->width;
+    			int i = x + y * matrix->width;
 
-    			matrixview_toggle_set_label( wid, matrix->value[pos] );
+    			matrixview_toggle_set_label( wid, matrix->value[i] );
     		}
 }
 
@@ -653,10 +666,10 @@ matrixview_slider_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			Tslider *tslider = TSLIDER( p->data );
-    			int pos = x + y * matrixview->width;
+    			int i = x + y * matrix->width;
 
-    			tslider->value = matrix->value[pos];
-    			tslider->svalue = matrix->value[pos];
+    			tslider->value = matrix->value[i];
+    			tslider->svalue = matrix->value[i];
 
     			tslider_changed( tslider );
     		}
@@ -688,7 +701,7 @@ matrixview_text_refresh( Matrixview *matrixview )
     	for( p = matrixview->items, y = 0; y < matrixview->height; y++ )
     		for( x = 0; x < matrixview->width; x++, p = p->next ) {
     			GtkWidget *item = GTK_WIDGET( p->data );
-    			int i = x + y * matrixview->width;
+    			int i = x + y * matrix->width;
     			double val = matrix->value[i];
 
     			matrixview_text_set( matrixview, item, val );
@@ -745,7 +758,36 @@ matrixview_refresh( vObject *vobject )
 {
     	Matrixview *matrixview = MATRIXVIEW( vobject );
     	Matrix *matrix = MATRIX( VOBJECT( matrixview )->iobject );
+
     	gboolean built = FALSE;
+    	gboolean hclip = FALSE;
+    	gboolean vclip = FALSE;
+	int width, height;
+	int i;
+
+	/* Find required size ... limit non-gtk_sheet displays to avoid huge
+	 * slowness.
+	 */
+	if( have_gtk_extra && 
+		(matrix->display == MATRIX_DISPLAY_TEXT || 
+		matrix->display == MATRIX_DISPLAY_TEXT_SCALE_OFFSET) ) {
+		width = matrix->width;
+		height = matrix->height;
+	}
+	else {
+		width = matrix->width;
+		height = matrix->height;
+
+		if( width > matrixview_max_width ) {
+			width = matrixview_max_width;
+			hclip = TRUE;
+		}
+
+		if( height > matrixview_max_height ) {
+			height = matrixview_max_height;
+			vclip = TRUE;
+		}
+	}
 
 #ifdef DEBUG
     	printf( "matrixview_refresh\n" );
@@ -755,21 +797,27 @@ matrixview_refresh( vObject *vobject )
     	 * type.
     	 */
     	if( matrixview->display != matrix->display || 
-    		matrixview->width != matrix->width || 
-    		matrixview->height != matrix->height ) {
+    		matrixview->width != width || 
+    		matrixview->height != height ) {
     		/* Kill old UI stuff.
     		 */
-    		FREEF( gtk_widget_destroy, matrixview->table );
-    		FREEF( gtk_widget_destroy, matrixview->sheet );
-    		FREEF( gtk_widget_destroy, matrixview->swin );
-    		FREEF( g_slist_free, matrixview->items );
-    		FREEF( gtk_widget_destroy, matrixview->cbox );
-		FREE( matrixview->cell_text );
+    		IM_FREEF( gtk_widget_destroy, matrixview->table );
+    		IM_FREEF( gtk_widget_destroy, matrixview->sheet );
+    		IM_FREEF( gtk_widget_destroy, matrixview->swin );
+    		IM_FREEF( g_slist_free, matrixview->items );
+    		IM_FREEF( gtk_widget_destroy, matrixview->cbox );
+		IM_FREE( matrixview->cell_text );
 		matrixview->cell_row = -1;
 		matrixview->cell_col = -1;
     		matrixview->scale = NULL;
     		matrixview->offset = NULL;
 
+		/* So the builders know how many widgets to make.
+		 */
+    		matrixview->width = width;
+    		matrixview->height = height;
+    		matrixview->display = matrix->display;
+
     		/* Make new contents. 
     		 */
 		switch( matrix->display ) {
@@ -790,9 +838,39 @@ matrixview_refresh( vObject *vobject )
 			assert( FALSE );
 		}
 
-    		matrixview->width = matrix->width;
-    		matrixview->height = matrix->height;
-    		matrixview->display = matrix->display;
+		if( hclip ) {
+			gtk_table_resize( GTK_TABLE( matrixview->table ), 
+				matrixview->height, matrixview->width + 1 );
+
+			for( i = 0; i < matrixview->height; i++ ) {
+				GtkWidget *lab;
+
+				lab = gtk_label_new( "---" );
+				gtk_table_attach( 
+					GTK_TABLE( matrixview->table ), lab,
+					matrixview->width, 
+					matrixview->width + 1, 
+					i, i + 1, 
+					GTK_FILL, GTK_FILL, 2, 2 );
+			}
+		}
+
+		if( vclip ) {
+			gtk_table_resize( GTK_TABLE( matrixview->table ), 
+				matrixview->height + 1, matrixview->width );
+
+			for( i = 0; i < matrixview->width; i++ ) {
+				GtkWidget *lab;
+
+				lab = gtk_label_new( "|" );
+				gtk_table_attach( 
+					GTK_TABLE( matrixview->table ), lab,
+					i, i + 1, 
+					matrixview->height, 
+					matrixview->height + 1, 
+					GTK_FILL, GTK_FILL, 2, 2 );
+			}
+		}
 
     		built = TRUE;
     	}
