@@ -488,7 +488,7 @@ program_find_pos( Program *program, const char *text, int *start, int *end )
 	else {
 		char *p;
 
-		if( (p = strcasestr( text, program->search )) ) {
+		if( (p = my_strcasestr( text, program->search )) ) {
 			*start = p - text;
 			*end = *start + strlen( program->search );
 
