@@ -523,32 +523,6 @@ reduce_get_matrix( Reduce *rc,
 	return( gmi.i );
 }
 
-/* Sub-fn of below. Test for 1st sz elements are char.
- */
-static void *
-reduce_test_char( Reduce *rc, PElement *base, int *sz )
-{
-	/* Tested enough?
-	 */
-	if( *sz == 0 ) 
-		return( NULL );
-
-	/* Reduce this list element.
-	 */
-	reduce_spine( rc, base );
-
-	/* Should be a char.
-	 */
-	if( !PEISCHAR( base ) ) 
-		return( base );
-
-	/* Move on.
-	 */
-	(*sz)--;
-
-	return( NULL );
-}
-
 /* Test for object is the empty list.
  */
 gboolean
@@ -573,18 +547,54 @@ reduce_islist( Reduce *rc, PElement *base )
 	return( FALSE );
 }
 
+/* Sub-fn of below. Test for 1st sz elements are char. We have several
+ * possible return values :-(
+ *
+ * - evaluation error ... we can throw an exception
+ * - we find a non-char in the first n elements ... return -1
+ * - we have tested the first n and want to stop looking ... return -2
+ * - all OK so far, but we want to keep looking ... return NULL
+ */
+static void *
+reduce_test_char( Reduce *rc, PElement *base, int *sz )
+{
+	/* Tested enough?
+	 */
+	if( *sz == 0 ) 
+		return( (void *) -2 );
+
+	/* Reduce this list element.
+	 */
+	reduce_spine( rc, base );
+
+	/* Should be a char.
+	 */
+	if( !PEISCHAR( base ) ) 
+		return( (void *) -1 );
+
+	/* Move on.
+	 */
+	(*sz)--;
+
+	return( NULL );
+}
+
 /* Test the first n elements of a list are char. n < 0 means test all
  * elements.
  */
 static gboolean
 reduce_n_isstring( Reduce *rc, PElement *base, int sz )
 {
+	void *result;
+
 	reduce_spine( rc, base );
 	if( !PEISLIST( base ) ) 
 		return( FALSE );
 
-	if( reduce_map_list( rc, base, 
-		(reduce_map_list_fn) reduce_test_char, &sz, NULL ) )
+	result = reduce_map_list( rc, base, 
+		(reduce_map_list_fn) reduce_test_char, &sz, NULL );
+
+	if( result == (void *) -1 )
 		return( FALSE );
 
 	return( TRUE );
