@@ -339,7 +339,8 @@ row_dirty_set( Row *row, gboolean clear_error )
 	row_dirty_set_single( row, clear_error );
 
 	return( icontainer_map_all( ICONTAINER( row ),
-		(icontainer_map_fn) row_dirty_set_sub, (void *) clear_error ) );
+		(icontainer_map_fn) row_dirty_set_sub, 
+			GINT_TO_POINTER( clear_error ) ) );
 }
 
 /* Break a dependency.
@@ -350,7 +351,7 @@ row_link_break( Row *parent, Row *child )
 	/* Must be there.
 	 */
 	assert( g_slist_find( parent->children, child ) &&
-	    g_slist_find( child->parents, parent ) );
+		g_slist_find( child->parents, parent ) );
 
 	parent->children = g_slist_remove( parent->children, child );
 	child->parents = g_slist_remove( child->parents, parent );
@@ -404,7 +405,7 @@ row_dispose( GObject *gobject )
 	if( row->top_row )
 		row->top_row->recomp_save = 
 			g_slist_remove( row->top_row->recomp_save, row );
-	FREEF( g_slist_free, row->recomp_save );
+	IM_FREEF( g_slist_free, row->recomp_save );
 
 	assert( !row->recomp );
 
@@ -466,7 +467,10 @@ row_get_column( Row *row )
 {
 	Subcolumn *scol = row_get_subcolumn( row );
 
-	return( scol->top_col ); 
+	if( scol )
+		return( scol->top_col ); 
+	else
+		return( NULL ); 
 }
 
 /* Search back up the widget hierarchy for the base row for this
@@ -486,7 +490,12 @@ row_get_root( Row *row )
 Workspace *
 row_get_workspace( Row *row )
 {
-	return( row_get_column( row )->ws );
+	Column *col = row_get_column( row );
+
+	if( col )
+		return( col->ws );
+	else
+		return( NULL );
 }
 
 static void
@@ -581,15 +590,20 @@ row_load( Model *model,
 	if( !MODEL_CLASS( parent_class )->load( model, state, parent, xnode ) )
 		return( FALSE );
 
+	/* If we've loaded a complete row system, mark this row plus any 
+	 * edited subrows dirty, and make sure this sym is dirty too.
+	 */
 	if( scol->istop ) {
-		/* If we've loaded a complete row system, mark this row plus
-		 * any edited subrows dirty, and make sure this sym is 
-		 * dirty too.
-		 */
 		row_dirty_set( row, TRUE );
 		expr_dirty( row->sym->expr, link_serial_new() );
 	}
 
+	/* If we've loaded a toplevel into a closed column, make sure display
+	 * is off.
+	 */
+	if( scol->istop && !scol->top_col->open )
+		model_set_display( MODEL( row ), FALSE );
+
 	return( TRUE );
 }
 
@@ -756,12 +770,18 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 #endif /*DEBUG*/
 
 	if( row_is_displayable( row->sym ) ) {
+		/* Hide superclasses whose constructor starts with "_".
+		 */
 		if( is_super( row->sym ) && PEISCLASS( root ) &&
 			*IOBJECT( PEGETCLASSCOMPILE( root )->sym )->name == 
 			'_' )
-			model_set_display( MODEL( heapmodel ), FALSE );
+			model_set_display( MODEL( row ), FALSE );
+		/* Hide top-level rows in closed columns.
+		 */
+		else if( row->scol->istop && !row->top_col->open )
+			model_set_display( MODEL( row ), FALSE );
 		else
-			model_set_display( MODEL( heapmodel ), TRUE );
+			model_set_display( MODEL( row ), TRUE );
 	}
 
 	/* New value ... reset error state.
@@ -772,7 +792,7 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	if( row->child_rhs &&
 		heapmodel_new_heap( HEAPMODEL( row->child_rhs ), root ) )
-		return( heapmodel );
+		return( row );
 
 	/* Class display only for non-param classes.
 	 */
@@ -786,7 +806,7 @@ row_new_heap( Heapmodel *heapmodel, PElement *root )
 		gboolean isclass;
 
 		if( !heap_isclass( root, &isclass ) )
-			return( heapmodel );
+			return( row );
 
 		/* If it's a class with a vis hint, use that.
 		 */
@@ -1190,7 +1210,7 @@ row_dirty( Row *row, gboolean clear_error )
 {
 	(void) row_dirty_set( row, clear_error );
 	(void) row_dependant_map( row, 
-		(row_map_fn) row_dirty_set, (void *) clear_error );
+		(row_map_fn) row_dirty_set, GINT_TO_POINTER( clear_error ) );
 
 	return( NULL );
 }
@@ -1202,7 +1222,7 @@ void *
 row_dirty_intrans( Row *row, gboolean clear_error )
 {
 	(void) row_dependant_map( row, 
-		(row_map_fn) row_dirty_set, (void *) clear_error );
+		(row_map_fn) row_dirty_set, GINT_TO_POINTER( clear_error ) );
 
 	return( NULL );
 }
@@ -1551,7 +1571,7 @@ row_recomp( Row *row )
 
 	/* Take a copy of the recomp list for later testing.
 	 */
-	FREEF( g_slist_free, top_row->recomp_save );
+	IM_FREEF( g_slist_free, top_row->recomp_save );
 	top_row->recomp_save = g_slist_copy( top_row->recomp );
 
 	/* Remove all top-level dependencies.
@@ -1590,7 +1610,7 @@ row_recomp( Row *row )
 	 * later.
 	 */
 	if( top_row->sym->ndirtychildren != 0 ) {
-		FREEF( g_slist_free, top_row->recomp_save );
+		IM_FREEF( g_slist_free, top_row->recomp_save );
 
 #ifdef DEBUG_TIME
 		printf( "row_recomp: delaying recomp of " );
@@ -1625,7 +1645,7 @@ row_recomp( Row *row )
 		row_recomp_all( top_row );
 	}
 
-	FREEF( g_slist_free, top_row->recomp_save );
+	IM_FREEF( g_slist_free, top_row->recomp_save );
 
 	symbol_dirty_clear( top_row->sym );
 
