@@ -230,22 +230,22 @@ rowview_update_widgets( Rowview *rview )
         /* Set colours.
          */
 	if( CALC_DISPLAY_LED ) {
-		LedColour colour;
+		char *stock_id;
 
+		stock_id = STOCK_LED_OFF;
 		if( row->selected )
-			colour = LED_GREEN;
+			stock_id = STOCK_LED_GREEN;
 		else if( row->show == ROW_SHOW_PARENT )
-			colour = LED_CYAN;
+			stock_id = STOCK_LED_CYAN;
 		else if( row->show == ROW_SHOW_CHILD )
-			colour = LED_BLUE;
+			stock_id = STOCK_LED_BLUE;
 		else if( row->err )
-			colour = LED_RED;
+			stock_id = STOCK_LED_RED;
 		else if( row->dirty )
-			colour = LED_YELLOW;
-		else
-			colour = LED_OFF;
+			stock_id = STOCK_LED_YELLOW;
 
-		led_set_colour( LED( rview->led ), colour ); 
+		gtk_image_set_from_stock( GTK_IMAGE( rview->led ), 
+			stock_id, GTK_ICON_SIZE_MENU );
 	}
 	else {
 		gchar *name = "";
@@ -742,8 +742,8 @@ rowview_init( Rowview *rview )
 
 	/* Make leds.
 	 */
-	rview->led = led_new();
-	led_set_type( LED( rview->led ), 1 );
+	rview->led = gtk_image_new_from_stock( STOCK_LED_OFF, 
+		GTK_ICON_SIZE_MENU );
 	gtk_misc_set_alignment( GTK_MISC( rview->led ), 0.5, 0.0 );
 	gtk_misc_set_padding( GTK_MISC( rview->led ), 2, 2 );
 
