@@ -45,7 +45,7 @@ static void *
 statusviewband_destroy_sub( StatusviewBand *svb )
 {
 	DESTROY_GTK( svb->val );
-	FREE( svb );
+	IM_FREE( svb );
 
 	return( NULL );
 }
@@ -57,7 +57,7 @@ statusviewband_destroy( Statusview *sv )
 {	
 	slist_map( sv->bands, 
 		(SListMapFn) statusviewband_destroy_sub, NULL );
-	FREEF( g_slist_free, sv->bands );
+	IM_FREEF( g_slist_free, sv->bands );
 }
 
 static void
