@@ -156,8 +156,8 @@ toolkitbrowser_entry_changed_cb( GtkEditable *editable,
 static gboolean
 toolkitbrowser_rebuild_test( Toolitem *toolitem, const char *text )
 {
-	if( strcasestr( toolitem->user_path, text ) || 
-		strcasestr( toolitem->tooltip, text ) )
+	if( my_strcasestr( toolitem->user_path, text ) || 
+		my_strcasestr( toolitem->tooltip, text ) )
 		return( TRUE );
 
 	return( FALSE );
