@@ -2323,8 +2323,12 @@ file_read_buffer( iOpenFile *of, char *buffer, size_t max )
 {
 	size_t len;
 
-	len = fread( buffer, sizeof( char ), max, of->fp );
+	/* -1 off max to leave space for the '\0'.
+	 */
+	len = fread( buffer, sizeof( char ), max - 1, of->fp );
 	if( !feof( of->fp ) ) {
+		/* Too large for buffer.
+		 */
 		of->last_errno = errno;
 		error_top( _( "Unable to read." ) );
 		error_sub( _( "Unable to read from file \"%s\".\n%s." ),
