@@ -201,9 +201,7 @@ gboolean isprefix( const char *a, const char *b );
 gboolean iscaseprefix( const char *a, const char *b );
 gboolean iscasepostfix( const char *a, const char *b );
 const char *findrightmost( const char *a, const char *b );
-#ifndef HAVE_STRCASESTR
-char *strcasestr( const char *haystack, const char *needle );
-#endif /*!HAVE_STRCASESTR*/
+char *my_strcasestr( const char *haystack, const char *needle );
 void change_suffix( const char *name, char *out, 
 	const char *new, const char **olds, int nolds );
 
