@@ -1483,6 +1483,17 @@ vips_fill_spine( VipsInfo *vi, HeapNode **arg )
 {
 	int i, j;
 
+	/* Fully reduce all arguments. Once we've done this, we can be sure
+	 * there will not be a GC while we gather, and therefore that no
+	 * pointers will become invalid during this call.
+	 */
+	for( i = 0; i < vi->nargs; i++ ) {
+		PElement rhs;
+
+		PEPOINTRIGHT( arg[i], &rhs );
+		reduce_spine_strict( vi->rc, &rhs );
+	}
+
 	for( j = 0, i = 0; i < vi->fn->argc; i++ ) {
 		im_arg_desc *varg = &vi->fn->argv[i];
 
