@@ -72,6 +72,8 @@ watchgroup_init( Watchgroup *watchgroup )
 #ifdef DEBUG
 	printf( "watchgroup_init\n" );
 #endif /*DEBUG*/
+
+	watchgroup->auto_save_timeout = 0;
 }
 
 GType
@@ -115,6 +117,92 @@ watchgroup_new( Workspacegroup *workspacegroup, const char *name )
 	return( watchgroup );
 }
 
+/* Get the ws we are storing prefs in, and check it looks OK.
+ */
+static Workspace *
+watchgroup_get_workspace( Watchgroup *watchgroup )
+{
+	Compile *compile;
+	Symbol *sym;
+
+	if( !watchgroup->workspacegroup->sym )
+		return( NULL );
+
+	compile = watchgroup->workspacegroup->sym->expr->compile;
+
+	if( !(sym = compile_lookup( compile, watchgroup->name )) ||
+		!sym->expr->compile ||
+		sym->type != SYM_WORKSPACE ||
+		!sym->ws )
+		return( NULL );
+
+	return( sym->ws );
+}
+
+static void
+watchgroup_save( Watchgroup *watchgroup )
+{
+	Workspace *ws;
+
+	if( (ws = watchgroup_get_workspace( watchgroup )) ) {
+		Filemodel *filemodel = FILEMODEL( ws );
+
+		if( filemodel->modified ) {
+			symbol_recalculate_all();
+
+			/* Ignore error returns ... hmm! Tricky: we can come
+			 * here during shutdown.
+			 */
+			(void) filemodel_save_all( filemodel, 
+				filemodel->filename );
+
+			filemodel_set_modified( filemodel, FALSE );
+		}
+	}
+}
+
+static gboolean
+watchgroup_dirty_timeout_cb( Watchgroup *watchgroup )
+{
+	watchgroup->auto_save_timeout = 0;
+
+	watchgroup_save( watchgroup );
+
+	return( FALSE );
+}
+
+void
+watchgroup_dirty( Watchgroup *watchgroup )
+{
+	Workspace *ws;
+
+	/* Find the preferences workspace.
+	 */
+	if( (ws = watchgroup_get_workspace( watchgroup )) ) {
+		Filemodel *filemodel = FILEMODEL( ws );
+
+		/* Mark ws dirty, start save timer.
+		 */
+		filemodel_set_modified( filemodel, TRUE );
+
+		IM_FREEF( g_source_remove, watchgroup->auto_save_timeout );
+		watchgroup->auto_save_timeout = g_timeout_add( 1000, 
+			(GSourceFunc) watchgroup_dirty_timeout_cb, watchgroup );
+	}
+}
+
+void
+watchgroup_flush( Watchgroup *watchgroup )
+{
+	/* Do we have a pending save?
+	 */
+	if( watchgroup->auto_save_timeout ) {
+		watchgroup_save( watchgroup );
+
+		IM_FREEF( g_source_remove, watchgroup->auto_save_timeout );
+	}
+}
+
 static iContainerClass *watch_parent_class = NULL;
 static GSList *watch_all = NULL;
 
@@ -273,30 +361,25 @@ watch_attach( Watch *watch )
 {
 	Watchgroup *watchgroup = WATCHGROUP( ICONTAINER( watch )->parent );
 	const char *name = IOBJECT( watch )->name;
-	Compile *compile;
-
-	Symbol *pref_ws, *pref_sym;
+	Workspace *ws;
+	Symbol *sym;
 
 	if( watch->row )
 		return;
-	if( !watchgroup->workspacegroup->sym )
-		return;
 
-	compile = watchgroup->workspacegroup->sym->expr->compile;
-
-	if( !(pref_ws = compile_lookup( compile, watchgroup->name )) ||
-		!pref_ws->expr->compile ||
-		!(pref_sym = compile_lookup( pref_ws->expr->compile, name )) ||
-		!pref_sym->expr->row )
-		return;
-
-	watch->row = pref_sym->expr->row;
-	watch->destroy_sid = 
-		g_signal_connect( G_OBJECT( watch->row ), "destroy",
-			G_CALLBACK( watch_destroy_cb ), watch );
-	watch->changed_sid = 
-		g_signal_connect( G_OBJECT( watch->row ), "changed",
-			G_CALLBACK( watch_changed_cb ), watch );
+	if( (ws = watchgroup_get_workspace( watchgroup )) &&
+		ws->sym->expr &&
+		ws->sym->expr->compile &&
+		(sym = compile_lookup( ws->sym->expr->compile, name )) &&
+		sym->expr->row ) {
+		watch->row = sym->expr->row;
+		watch->destroy_sid = 
+			g_signal_connect( G_OBJECT( watch->row ), "destroy",
+				G_CALLBACK( watch_destroy_cb ), watch );
+		watch->changed_sid = 
+			g_signal_connect( G_OBJECT( watch->row ), "changed",
+				G_CALLBACK( watch_changed_cb ), watch );
+	}
 }
 
 Watch *
@@ -347,6 +430,40 @@ watch_relink_all( void )
 	slist_map( watch_all, (SListMapFn) watch_relink, NULL );
 }
 
+void
+watch_vset( Watch *watch, const char *fmt, va_list args )
+{
+	if( watch->row && watch->row->child_rhs && 
+		watch->row->child_rhs->itext ) {
+		iText *itext = ITEXT( watch->row->child_rhs->itext );
+		char buf[256];
+
+		(void) im_vsnprintf( buf, 256, fmt, args );
+
+		if( itext_set_formula( itext, buf ) ) {
+#ifdef DEBUG
+			printf( "watch_vset: %s = %s\n", 
+				IOBJECT( watch )->name, buf );
+#endif /*DEBUG*/
+
+			itext_set_edited( itext, TRUE );
+			expr_dirty( watch->row->sym->expr, link_serial_new() );
+			watchgroup_dirty( 
+				WATCHGROUP( ICONTAINER( watch )->parent ) );
+		}
+	}
+}
+
+void
+watch_set( Watch *watch, const char *fmt, ... )
+{
+	va_list args;
+
+	va_start( args, fmt );
+	watch_vset( watch, fmt, args );
+	va_end( args );
+}
+
 static WatchClass *watch_double_parent_class = NULL;
 
 static gboolean
@@ -550,7 +667,7 @@ watch_int_get( Watchgroup *watchgroup, const char *name, int fallback )
 	Watch *watch;
 	void *value;
 
-	if( !watchgroup )
+	if( !watchgroup || !name )
 		return( fallback );
 
 	if( !(watch = watch_find( watchgroup, name )) )
@@ -582,7 +699,7 @@ watch_path_finalize( GObject *gobject )
 
 	/* My instance destroy stuff.
 	 */
-	FREEF( slist_free_all, watch_path->value );
+	IM_FREEF( slist_free_all, watch_path->value );
 
 	G_OBJECT_CLASS( watch_path_parent_class )->finalize( gobject );
 }
@@ -602,7 +719,7 @@ watch_path_update( Watch *watch )
 	if( !heap_get_lstring( root, &value ) )
 		return( FALSE );
 
-	FREEF( slist_free_all, watch_path->value );
+	IM_FREEF( slist_free_all, watch_path->value );
 	watch_path->value = value;
 
 	return( TRUE );
@@ -823,7 +940,7 @@ watch_string_finalize( GObject *gobject )
 
 	/* My instance destroy stuff.
 	 */
-	FREE( watch_string->value );
+	IM_FREE( watch_string->value );
 
 	G_OBJECT_CLASS( watch_string_parent_class )->finalize( gobject );
 }
