@@ -107,6 +107,11 @@ struct _Workspace {
 	 * picked up by them.
 	 */
 	iWindow *iwnd;
+
+	/* Visualisation defaults for this ws.
+	 */
+	double scale;
+	double offset;
 };
 
 typedef struct _WorkspaceClass {
@@ -147,6 +152,7 @@ Column *workspace_column_pick( Workspace *ws );
 void workspace_column_select( Workspace *ws, Column *col );
 
 Symbol *workspace_add_def( Workspace *ws, const char *str );
+Symbol *workspace_load_file( Workspace *ws, const char *filename );
 
 gboolean workspace_selected_save( Workspace *ws, const char *filename );
 gboolean workspace_clone_selected( Workspace *ws );
